
_ = require('lodash')

defaults = {
  client: null
  prefix: 'redlocks:'
  timeout: 60
  retries: 2
}

RELEASE_LUA = """
  if redis.call("get",KEYS[1]) == ARGV[1] then
      return redis.call("del",KEYS[1])
  else
      return 0
  end
"""

KEEPALIVE_LUA = """
  if redis.call("get",KEYS[1]) == ARGV[1] then
      return redis.call("expire",ARGV[2])
  else
      return 0
  end
"""

redlock = (opts) ->

  opts = _.merge(defaults, opts)

  unless opts.client?
    throw new Error("You must provide a Redis client to redlock")

  return (name, timeout, callback) ->
    unless callback?
      callback = timeout
      timeout = opts.timeout
    unless callback?
      callback = _.noop
    }

    lockId = opts.prefix + name
    token = "#{Date.now()}-#{(Math.random() + 1).toString(36).substr(2, 9)}"

    lock = {
      release: (callback) ->
        callback ?= _.noop
        opts.client.eval([RELEASE_LUA, 1, lockId, token], callback)
      }
      keepAlive: (callback) ->
        callback ?= _.noop
        opts.client.eval([KEEPALIVE_LUA, 1, lockId, token, timeout], callback)
      }
    }

    opts.client.set([lockId, token, 'NX', 'EX', timeout], (err, value) ->
      if err
        return callback(err)

      if value
        callback(null, lock)
      else
        callback()
    )

module.exports = redlock
